/*  TI Device Driver for Linux                                       *
 *  Allows the homebuilt TI parallel link cable                      *
 *  to funcion with the linux and windows link                       *
 *  software made for the TI-GraphLink.                              *
 *                                                                   *
 *  David Kuder (xwmchooser@email.com)                               *
 *  Based on TI-TERM by Michael Katz-Hyman (mkatshym@erols.com)      *
 *  Parts taken from ti.c by David Ellsworth                         *
 *  Parts taken from Dos Term by Brian Rittmyer(sp?)                 */
 
#include <stdio.h>
#include <asm/io.h>
#include <termios.h>
#include <fcntl.h>
#include <posix_types.h>

#define TIMAXTIME 10000
#define IO_DELAY  50 
#define lpt_base  0x378
  
int lpt_out=lpt_base;
int lpt_in=(lpt_base+1);
      
static char *fifo_name, *prog_name;
static int fifo, serialport;

int init_ti();
int put_ti(unsigned char data);
int probe();
int get_ti();
void serial_output();
void init_keyboard();
void close_keyboard();
int kbdhit();
int readch();
    
/* Some stuff for the kbdhit() routine */

static struct termios initial_settings, new_settings;
static int peek_character = -1;

int main(int argc, char **argv) {
  int stop=0, key, ret;
  
  fprintf(stdout, "TI-Parport Device driver v1.0\n");
  fprintf(stdout, "Based on TI-Term by Michael Katz-Hyman (mkatshym@erols.com).\n"); 

  prog_name = argv[0];
  if (argc < 2) {
     fprintf(stderr, "Usage: %s <device>\n", argv[0]);
     exit(1);
  }

  fifo_name = argv[1];

  fprintf(stdout, "Using %s for I/O.\n", fifo_name);
  fprintf(stdout, "Press ESC to exit.\n");
  /* Ask permision to use the ports */
  
  if (ioperm(lpt_base,3,1)) {perror("ioperm");exit(1);}

  /* Initiate the terminal routines */
  
    init_keyboard();
    
  /* Initiate the serial port */

  serialport = open(fifo_name, O_RDWR);

  /* Initiate the link port */

  init_ti(); 
  
  /* Let's get this party started */

  while (stop !=1) {

    if (kbdhit()) {
      key = readch();
      if (key == 27) {
	stop = 1;
	continue;
      }
    }

    check_ser();
    continue;

    ret = get_ti();
    if (ret == -1) continue;
    serial_output(ret, 0);
  
  }
  /* We don't need the ports anymore */

  if (ioperm(lpt_base,3,0)) {perror("ioperm");exit(1);}

  /* Or the keyboard */
  
    close_keyboard();
    
  fflush(stdin);
  return 0;
}

int init_ti() {
  outb(3, lpt_out);
  return 0;
}

int put_ti(unsigned char data) {
  int bit,i,titime=0;

  for (bit=0; bit<8; bit++) {
    for(i=0; i<IO_DELAY; i++) inb(lpt_in);
    
    if (data&1) {
      outb(2, lpt_out);
      while (inb(lpt_in)&0x10) if(titime++>TIMAXTIME) return -1;
      outb(3, lpt_out);
      while (!(inb(lpt_in)&0x10)) if(titime++>TIMAXTIME) return -1;
    } else {
      outb(1, lpt_out);
      while (inb(lpt_in)&0x20) if(titime++>TIMAXTIME) return -1;
      outb(3, lpt_out);
      while (!(inb(lpt_in)&0x20)) if(titime++>TIMAXTIME) return -1;
    }
    data>>=1;
  }
  return;
}

int get_ti() {
  int bit,i,titime=0;
  unsigned char data=0, v;

  for (bit=0; bit<8; bit++) {
    while ((v=inb(lpt_in)&0x30)==0x30) if(titime++>TIMAXTIME) return -1;
    if (v==0x10) {
      data=(data>>1)|0x80;
      outb(1, lpt_out);
      while (!(inb(lpt_in)&0x20)) if(titime++>TIMAXTIME) return -1;
      outb(3, lpt_out);
    } else {
      data=data>>1;
      outb(2, lpt_out);
      while (!(inb(lpt_in)&0x10)) if(titime++>TIMAXTIME) return -1;
      outb(3, lpt_out);
    }
    for(i=0; i<IO_DELAY; i++) inb(lpt_in);
    
  }
  return (int)data;
}

void serial_output(s, len)
char *s;
int len;
{

  char buf[256];

  if (len == 0) len = strlen(s);
	write(serialport, s, len);

}

unsigned char getbyte(void)
{
static unsigned char buf[1024];
static unsigned char *bp = buf, *ep = buf;
int n;

if (bp == ep) {
    bp = ep = buf;
    n = read(serialport, buf, 1024);
    if (n>0) {
      ep += n;
    }
  }
  return *bp++;
}

int serial_input(filedesc, buf, buflen)
int filedesc;
char *buf[8];
int *buflen;
{
  int n = 0, i;
  fd_set fds;

  i = filedesc;

  __FD_ZERO(&fds);
  if (filedesc >= 0) __FD_SET(filedesc, &fds); else filedesc = 0;


  /* If there is data put it in the buffer. */
  if (buf) {
        i = 0;
        if ((n & 1) == 1) i = read(filedesc, buf, 1);
        buf[i > 0 ? i : 0] = 0;
        if (buflen) *buflen = i;
  }

  return(n);
}

int check_ser (serialport)
int serialport;
{
  char buf[8];
  char *ptr;
  int x;
  int blen;

        x = serial_input(serialport, buf, &blen);

        /*  Send data from the serialport to the calc. */
        if ((x & 1) == 1) {
                ptr = buf;
                if (blen == 1) {
                        put_ti(*ptr++);
                        blen = 0;
                }
        }
};

void init_keyboard() {

  tcgetattr(0,&initial_settings);
  new_settings = initial_settings;
  new_settings.c_lflag &= ~ICANON;
  new_settings.c_lflag &= ~ECHO;
  new_settings.c_lflag &= ~ISIG;
  new_settings.c_cc[VMIN] = 1;
  new_settings.c_cc[VTIME] = 0;
  tcsetattr(0, TCSANOW, &new_settings);

}

void close_keyboard() {

  tcsetattr(0,TCSANOW, &initial_settings);
}

int kbdhit() {

  char ch;
  int nread;

  if(peek_character != -1)
    return 1;
  new_settings.c_cc[VMIN]=0;
  tcsetattr(0, TCSANOW, &new_settings);
  nread = read(0, &ch, 1);
  new_settings.c_cc[VMIN]=1;
  tcsetattr(0, TCSANOW, &new_settings);

  if(nread == 1) {
    peek_character = ch;
    return 1;
  }
  return 0;
}

int readch() {

  char ch;

  if(peek_character != -1) {
    ch = peek_character;
    peek_character = -1;
    return ch;
  }
  read(0,&ch,1);
  return ch;
}
